/*****************************************************************************
  FILE           : $Source: /projects/higgs1/SNNS/CVS/SNNS/xgui/sources/ui_xWidgets.h,v $
  SHORTNAME      : xWidgets 
  SNNS VERSION   : 4.2

  PURPOSE        : Private Header file of correspondent '.c' file
  NOTES          :

  AUTHOR         : Tilman Sommer 
  DATE           : 13.7.1990

  CHANGED BY     : Sven Doering
  RCS VERSION    : $Revision: 2.8 $
  LAST CHANGE    : $Date: 1998/02/25 15:22:55 $

    Copyright (c) 1990-1995  SNNS Group, IPVR, Univ. Stuttgart, FRG
    Copyright (c) 1996-1998  SNNS Group, WSI, Univ. Tuebingen, FRG

******************************************************************************/

/*Bool     ui_xIsRealized();*/
extern void     ui_xSetResize(Widget widget,Boolean state);
extern void     ui_xSetLabel(Widget widget,char *newLabel);
extern void     ui_xSetString(Widget widget,char *contents);
extern void     ui_xStringFromAsciiWidget(Widget widget,char *stringVar,int maxLen);
extern float    ui_xFloatFromAsciiWidget(Widget widget);
extern int      ui_xIntFromAsciiWidget(Widget widget);
extern void     ui_xGetDimensions(Widget widget,Dimension *width,Dimension *height);
extern Bool     ui_xGetToggleState(Widget w);
extern void     ui_xSetToggleState(Widget w,Bool state);
extern void     ui_setToggles(int index, int *new, int max, Widget widgets[]);
extern void     ui_xAddListEntry(struct SimpleListType *listDescriptorPtr,String string);
extern void ui_checkWindowPosition(Widget w);

extern Widget   ui_xCreateLabelItem(char *name,Widget parent,Dimension width,Widget left,Widget top);
extern Widget   ui_xCreateFrameLabelItem(char *name,Widget parent,Dimension width,Widget left,Widget top);

extern Widget ui_xCreateFrameLabelItemCentered (char *name, Widget parent,
						Dimension width,
						Widget left, Widget top);

extern Widget   ui_xCreateDialogItem(char *name,Widget parent,char *contents,Dimension  width,Widget left,Widget top);     /* AsciiText string edit */
extern Widget   ui_xCreateButtonItem(char *name,Widget parent,Widget left,Widget top);
extern Widget   ui_xCreateMenuButtonItem(char *name,Widget parent,Widget left,Widget top);
extern Widget   ui_xCreateToggleItem(char *name,Widget  parent,Widget  group,Widget  left,Widget  top);
extern Widget   ui_xCreateScrollbarItem(char *name,Widget parent,Dimension length,float topOfThumb,Widget left,Widget top);
extern void ui_xDontResizeWidget (Widget w);





/* end of file */
/* lines: */





/* 52 lines generated by deleteprivatedefinitions.awk */
